//
// C++ <-> Lua expoter
// Magnus sterlind (magnus@dooz.se)
//

#include "LuaExporter.hpp"
#include <string>
#include <assert.h>

using namespace std;
// The lua file we're testing
/*
-- test calling int function
test_int(42)

-- test calling double
test_double(1.23)

-- test calling string
test_string("hello!")

-- test multiple values
test_multiple(1, 2.34, "5.678")

-- test return values
test_return( test_inc_int(42) )

-- test with member functions
test_member_int(42)
test_member_double(1.23)
test_member_string("hello!")
test_member_multiple(1, 2.34, "5.678")
test_member_return( test_inc_int(42) )
*/

void test_int(const int value) {
  assert(value == 42);    
}

void test_double(const double value) {
  assert(value == 1.23);
}

void test_string(const string& value) {
  assert(value == "hello!");
}

void test_multiple(const int int_value, const double double_value, const string& string_value) {
  assert(int_value == 1);
  assert(double_value == 2.34);
  assert(string_value == "5.678");
}

int test_inc_int(const int value) {
  return value + 1;
}

void test_return(const int value) {
  assert(value == 43);
}

struct Test{
  void test_member_int(const int value) {
    assert(value == 42);    
  }

  void test_member_double(const double value) {
    assert(value == 1.23);
  }

  void test_member_string(const string& value) {
    assert(value == "hello!");
  }

  void test_member_multiple(const int int_value, const double double_value, const string& string_value) {
    assert(int_value == 1);
    assert(double_value == 2.34);
    assert(string_value == "5.678");
  }

  void test_member_return(const int value) {
    assert(value == 43);
  }
};


int main() {
  LuaExporter exporter;

  // add free functions
  exporter.Register("test_int", &test_int);
  exporter.Register("test_double", &test_double);
  exporter.Register("test_string", &test_string);
  exporter.Register("test_multiple", &test_multiple);
  exporter.Register("test_inc_int", &test_inc_int);
  exporter.Register("test_return", &test_return);

  // add member functions
  Test t;
  exporter.Register("test_member_int", &Test::test_member_int, &t);
  exporter.Register("test_member_double", &Test::test_member_double, &t);
  exporter.Register("test_member_string", &Test::test_member_string, &t);
  exporter.Register("test_member_multiple", &Test::test_member_multiple, &t);
  exporter.Register("test_member_return", &Test::test_member_return, &t);

  exporter.DoFile("test.lua");

  return 0;
}

